<?php
// TTFReader Class
// Version 1.0.0
// Copyright (C) 2003, Steve Blinch
// http://code.blitzaffe.com
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Ported from C++ code by Philip Patrick.  Original C++ code available at:
// http://www.codeproject.com/gdi/fontnamefromfile.asp

define("SIZE_USHORT",2);
define("SIZE_ULONG",4);

define("PLATFORM_APPLE",0);		// Apple unicode
define("PLATFORM_MAC",1);		// Macintosh
define("PLATFORM_ISO",2);		// ISO encoding
define("PLATFORM_WINDOWS",3);	// Microsoft

define("LANGUAGE_ENG_US",1033);	// US/English


class TTFReader {
	
	function TTFReader() {
		/* 
		// TTF file header
		typedef struct _tagTT_OFFSET_TABLE{
		    USHORT uMajorVersion;
		    USHORT uMinorVersion;
		    USHORT uNumOfTables;
		    USHORT uSearchRange;
		    USHORT uEntrySelector;
		    USHORT uRangeShift;
		}TT_OFFSET_TABLE;
		
		// Table directory
		typedef struct _tagTT_TABLE_DIRECTORY{
		    char szTag[4]; //table name
		    ULONG uCheckSum; //Check sum
		    ULONG uOffset; //Offset from beginning of file
		    ULONG uLength; //length of the table in bytes
		}TT_TABLE_DIRECTORY;
		
		// Names table header
		typedef struct _tagTT_NAME_TABLE_HEADER{
		    USHORT uFSelector; //format selector. Always 0
		    USHORT uNRCount; //Name Records count
		    USHORT uStorageOffset; //Offset for strings storage, 
		                           //from start of the table
		}TT_NAME_TABLE_HEADER;
		
		// Names table record
		typedef struct _tagTT_NAME_RECORD{
		    USHORT uPlatformID;
		    USHORT uEncodingID;
		    USHORT uLanguageID;
		    USHORT uNameID;
		    USHORT uStringLength;
		    USHORT uStringOffset; //from start of storage area
		}TT_NAME_RECORD;		
				
		*/
		// define our record structures
		$this->structs = array(
			"TT_OFFSET_TABLE" => array(
				"struct"=>"nMajorVersion/nMinorVersion/nNumOfTables/nSearchRange/nEntrySelector/nRangeShift",
				"size"=>SIZE_USHORT*6
			),		
			"TT_TABLE_DIRECTORY" => array(
				"struct"=>"C4Tag/NCheckSum/NOffset/NLength",
				"size"=>4 + SIZE_ULONG*3
			),
			"TT_NAME_TABLE_HEADER" => array(
				"struct"=>"nFSelector/nNRCount/nStorageOffset",
				"size"=>SIZE_USHORT*3
			),
			"TT_NAME_RECORD" => array(
				"struct"=>"nPlatformID/nEncodingID/nLanguageID/nNameID/nStringLength/nStringOffset",
				"size"=>SIZE_USHORT*6
			)
		);
	}
	
	// read a structure from a file
	function bread(&$fp,$struct_key) {
		$struct = &$this->structs[$struct_key];
		$data = fread($fp,$struct["size"]);
		return @unpack($struct["struct"],$data);
	}
	
	// really cheezy/slow way to convert a multibyte string (we can't really
	// assume that the multibyte PHP extensions have been compiled in)
	function strip_multibyte($s) {
		$r = "";
		for ($i=0; $i<strlen($s); $i+=2) $r.=$s[$i+1];
		return $r;
	}
	
	// read name info from a file
	function get($filepath,$platform=PLATFORM_WINDOWS,$language=LANGUAGE_ENG_US) {
		$this->font = array();
		$nameidlist = array(
			0=>"copyright",
			1=>"familyname",
			2=>"style",
			3=>"fontid",
			4=>"fullname",
			5=>"version",
			6=>"psname",
			7=>"trademark",
			8=>"manufacturer",
			9=>"designer",
			10=>"description",
			11=>"vendorurl",
			12=>"designerurl",
			13=>"license",
			14=>"licenseurl",
			19=>"sampletext"
		);
	
		$this->filepath = $filepath;
		if ($fp = fopen($filepath,"rb")) {
			
			if (!$ttOffsetTable = $this->bread($fp,"TT_OFFSET_TABLE")) {
				$this->error = "Invalid TTF file";
				fclose($fp);
				return false;
			}
	
			if ($ttOffsetTable["MajorVersion"] != 1 || $ttOffsetTable["MinorVersion"] != 0) {
				$this->error = "Invalid TTF version";
				fclose($fp);
				return false;
			}
			$bFound = FALSE;
			
			for ($i=0; $i<$ttOffsetTable["NumOfTables"]; $i++) {
				$tblDir = $this->bread($fp,"TT_TABLE_DIRECTORY");
				
				$tbltag = "";
				for ($j=1; $j<5; $j++) $tbltag .= chr($tblDir["Tag$j"]);
	
				if($tbltag=="name") {
					$bFound = TRUE;
					break;
				}
			}
			if($bFound){
				fseek($fp,$tblDir["Offset"]);
				$ttNTHeader = $this->bread($fp,"TT_NAME_TABLE_HEADER");
	
				$bFound = false;
				
				for($i=0; $i<$ttNTHeader["NRCount"]; $i++) {
					$ttRecord = $this->bread($fp,"TT_NAME_RECORD");

					if($ttRecord["PlatformID"]==$platform && $ttRecord["LanguageID"]==$language) {
						$nPos = ftell($fp);
						fseek($fp,$tblDir["Offset"] + $ttRecord["StringOffset"] + $ttNTHeader["StorageOffset"]);
	
						$NameBuf = fread($fp,$ttRecord["StringLength"]);
						if ($NameBuf) {
							$NameBuf = $this->strip_multibyte($NameBuf);
							
							$nameid = $ttRecord["NameID"];
							if (isset($nameidlist[$nameid])) {
								$this->font[$nameidlist[$nameid]] = $NameBuf;
								$bFound = true;
							}
						}
						
						fseek($fp,$nPos);
					}
				}			
			} else {
				$this->error = "TTF name record not found";
			}
			fclose($fp);
		}
		return $bFound;
	}
	
	function get_all($directory,$cachefile=false,$platform=PLATFORM_WINDOWS,$language=LANGUAGE_ENG_US) {
		if ($cachefile) {
			$cache = @implode("",@file($cachefile));
			$cache = unserialize(base64_decode($cache));
		} else {
			$cache = false;
		}
		
		if(substr($directory,-1)!="/") $directory .= "/";
		$d = dir($directory); 
		$fonts = array();
		while (false !== ($entry = $d->read())) { 
			$newfont = false;
			
			if (is_array($cache) && isset($cache[$entry])) {
				$newfont = $cache[$entry];
			} else {
				if (substr($entry,0,1)==".") continue;
				if (is_dir($directory.$entry)) continue;
				if ($this->get($directory.$entry,$platform)) {
					$newfont = $this->font;
				}
			}
			if ($newfont) {
				$newfont["filename"] = $entry;
				$fonts[$entry] = $newfont;
			}
		}
		$d->close();
		
		if ($cachefile) {
			$cache = base64_encode(serialize($fonts));
			$fp = @fopen($cachefile,"w");
			if ($fp) {
				fwrite($fp,$cache);
				fclose($fp);
			}
		}
		return $fonts;
	}
}
?>
